﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Autodesk.AutoCAD.Runtime;
using ACSMCOMPONENTS18Lib;

namespace SheetSetManagerCsharp
{
    public class ADSK_SheetSet 
    { 
        MySSmEventHandler eventHandler; 
        Int32 eventSSMCookie;
        Int32 eventDbCookie;
        Int32 eventSSetCookie; 
        IAcSmSheetSetMgr m_sheetSetManager; 
        IAcSmDatabase m_sheetSetDatabase; 
        IAcSmSheetSet m_sheetSet; 
        
        // Open a Sheet Set 
        [CommandMethod("ADSK_OpenSheetSet")] 
        public void OpenSheetSet() 
        { 
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = default(IAcSmSheetSetMgr); 
            sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Open a Sheet Set file 
            AcSmDatabase sheetSetDatabase = default(AcSmDatabase); 
            sheetSetDatabase = sheetSetManager.OpenDatabase("C:\\Program Files\\AutoCAD 2010\\Sample\\Sheet Sets\\Architectural\\IRD Addition.dst", false); 
            
            // Return the namd and description of the sheet set
            MessageBox.Show("Sheet Set Name: " + sheetSetDatabase.GetSheetSet().GetName() + "\nSheet Set Description: " + sheetSetDatabase.GetSheetSet().GetDesc());
            
            // Close the sheet set 
            sheetSetManager.Close(sheetSetDatabase); 
        } 
        
        // Create a new sheet set 
        [CommandMethod("ADSK_CreateSheetSet")] 
        public void CreateSheetSet() 
        { 
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Create a new sheet set file 
            AcSmDatabase sheetSetDatabase = sheetSetManager.CreateDatabase("C:\\Datasets\\CP318-4\\CP318-4.dst", "", true); 
            
            // Get the sheet set from the database 
            AcSmSheetSet sheetSet = sheetSetDatabase.GetSheetSet(); 
            
            // Set the name and description of the sheet set 
            sheetSet.SetName("CP318-4"); 
            sheetSet.SetDesc("AU2009 Sheet Set Object Demo"); 
            
            // Return the name and description of the sheet set 
            MessageBox.Show("Sheet Set Name: " + sheetSetDatabase.GetSheetSet().GetName() + "\nSheet Set Description: " + sheetSetDatabase.GetSheetSet().GetDesc());
            
            // Close the sheet set 
            sheetSetManager.Close(sheetSetDatabase); 
        } 
        
        // Create a new sheet set 
        [CommandMethod("ADSK_CreateSheetSet_LockExample")] 
        public void CreateSheetSet_LockExample() 
        { 
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Create a new sheet set file 
            AcSmDatabase sheetSetDatabase = sheetSetManager.CreateDatabase("C:\\Datasets\\CP318-4\\CP318-4.dst", "", true); 
            
            // Get the sheet set from the database 
            AcSmSheetSet sheetSet = sheetSetDatabase.GetSheetSet(); 
            
            // Attempt to lock the database
            if (LockDatabase(ref sheetSetDatabase, true) == true) { 
                // Set the name and description of the sheet set 
                sheetSet.SetName("CP318-4"); 
                sheetSet.SetDesc("AU2009 Sheet Set Object Demo"); 
                
                // Unlock the database 
                LockDatabase(ref sheetSetDatabase, false); 
                
                // Return the name and description of the sheet set 
                MessageBox.Show("Sheet Set Name: " + sheetSetDatabase.GetSheetSet().GetName() + "\nSheet Set Description: " + sheetSetDatabase.GetSheetSet().GetDesc()); 
            } 
            else { 
                // Display error message 
                MessageBox.Show("Sheet set could not be opened for write."); 
            } 
            
            // Close the sheet set 
            sheetSetManager.Close(sheetSetDatabase); 
        } 
        
        // Step through all open sheet sets 
        [CommandMethod("ADSK_StepThroughTheOpenSheetSets")] 
        public void StepThroughTheOpenSheetSets() 
        { 
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Get the loaded databases 
            IAcSmEnumDatabase enumDatabase = sheetSetManager.GetDatabaseEnumerator(); 
            
            // Get the first open database 
            IAcSmPersist item = enumDatabase.Next(); 
            
            string customMessage = ""; 
            
            // If a database is open continue 
            if ((item != null)) { 
                int count = 0; 
                
                // Step through the database enumerator 
                while ((item != null)) { 
                    // Append the file name of the open sheet set to the output string 
                    customMessage = customMessage + "\n" + item.GetDatabase().GetFileName(); 
                    
                    // Get the next open database and increment the counter 
                    item = enumDatabase.Next(); 
                    count = count + 1; 
                } 
                
                customMessage = "Sheet sets open: " + count.ToString() + customMessage; 
            } 
            else { 
                customMessage = "No sheet sets are currently open."; 
            } 
            
            // Display the custom message 
            MessageBox.Show(customMessage); 
        } 
        
        // Create a new sheet set 
        [CommandMethod("ADSK_CreateSheetSet_AddSubset")] 
        public void CreateSheetSet_AddSubset() 
        { 
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Create a new sheet set file 
            AcSmDatabase sheetSetDatabase = sheetSetManager.CreateDatabase("C:\\Datasets\\CP318-4\\CP318-4.dst", "", true); 
            
            // Get the sheet set from the database 
            AcSmSheetSet sheetSet = sheetSetDatabase.GetSheetSet(); 
            
            // Attempt to lock the database 
            if (LockDatabase(ref sheetSetDatabase, true) == true) { 
                // Set the name and description of the sheet set 
                sheetSet.SetName("CP318-4"); 
                sheetSet.SetDesc("AU2009 Sheet Set Object Demo"); 
                
                // Create two new subsets 
                AcSmSubset subset = default(AcSmSubset); 
                subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", false); 
                
                subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", true); 
                
                // Unlock the database 
                LockDatabase(ref sheetSetDatabase, false); 
            } 
            else { 
                // Display error message 
                MessageBox.Show("Sheet set could not be opened for write."); 
            } 
            
            // Close the sheet set 
            sheetSetManager.Close(sheetSetDatabase); 
        } 
        
        // Create a new sheet set 
        [CommandMethod("ADSK_CreateSheetSet_AddSheet")] 
        public void CreateSheetSet_AddSheet() 
        { 
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Create a new sheet set file 
            AcSmDatabase sheetSetDatabase = sheetSetManager.CreateDatabase("C:\\Datasets\\CP318-4\\CP318-4.dst", "", true); 
            
            // Get the sheet set from the database 
            AcSmSheetSet sheetSet = sheetSetDatabase.GetSheetSet(); 
            
            // Attempt to lock the database 
            if (LockDatabase(ref sheetSetDatabase, true) == true) { 
                // Get the folder the sheet set is stored in 
                string sheetSetFolder = sheetSetDatabase.GetFileName().Substring(0, sheetSetDatabase.GetFileName().LastIndexOf("\\"));
                
                // set the default values of the sheet set 
                SetSheetSetDefaults(sheetSetDatabase, "CP318-4", "AU2009 Sheet Set Object Demo", sheetSetFolder, "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet"); 
                
                AddSheet(sheetSetDatabase, "Title Page", "Project Title Page", "Title Page", "T1"); 
                
                // Create two new subsets 
                AcSmSubset subset = default(AcSmSubset); 
                subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", false); 
                
                AddSheet(subset, "North Plan", "Northern section of building plan", "North Plan", "P1"); 
                
                subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", true); 
                
                // Unlock the database 
                LockDatabase(ref sheetSetDatabase, false); 
            } 
            else { 
                // Display error message 
                MessageBox.Show("Sheet set could not be opened for write."); 
            } 
            
            // Close the sheet set 
            sheetSetManager.Close(sheetSetDatabase); 
        } 
        
        // Create a new sheet set 
        [CommandMethod("ADSK_CreateSheetSet_AddCustomProperty")] 
        public void CreateSheetSet_AddCustomProperty() 
        { 
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Create a new sheet set file 
            AcSmDatabase sheetSetDatabase = sheetSetManager.CreateDatabase("C:\\Datasets\\CP318-4\\CP318-4.dst", "", true); 
            
            // Get the sheet set from the database 
            AcSmSheetSet sheetSet = sheetSetDatabase.GetSheetSet(); 
            
            // Attempt to lock the database 
            if (LockDatabase(ref sheetSetDatabase, true) == true) { 
                // Get the folder the sheet set is stored in 
                string sheetSetFolder = null;
                sheetSetFolder = sheetSetDatabase.GetFileName().Substring(0, sheetSetDatabase.GetFileName().LastIndexOf("\\"));
                
                // Set the default values of the sheet set 
                SetSheetSetDefaults(sheetSetDatabase, "CP318-4", "AU2009 Sheet Set Object Demo", sheetSetFolder, "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet"); 
                
                // Create a sheet set property 
                SetCustomProperty(sheetSet, "Project Approved By", "AU09", PropertyFlags.CUSTOM_SHEETSET_PROP); 
                
                // Create sheet properties 
                SetCustomProperty(sheetSet, "Checked By", "LAA", PropertyFlags.CUSTOM_SHEET_PROP);

                SetCustomProperty(sheetSet, "Complete Percentage", "0%", PropertyFlags.CUSTOM_SHEET_PROP); 
                
                AddSheet(sheetSetDatabase, "Title Page", "Project Title Page", "Title Page", "T1"); 
                
                // Create two new subsets 
                AcSmSubset subset = default(AcSmSubset); 
                subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", false); 
                
                AddSheet(subset, "North Plan", "Northern section of building plan", "North Plan", "P1"); 
                
                subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", true); 
                
                // Unlock the database 
                LockDatabase(ref sheetSetDatabase, false); 
            } 
            else { 
                // Display error message 
                MessageBox.Show("Sheet set could not be opened for write."); 
            } 
            
            // Close the sheet set 
            sheetSetManager.Close(sheetSetDatabase); 
        } 
        
        // Create a new sheet set 
        [CommandMethod("ADSK_CreateSheetSet_SyncProperties")] 
        public void CreateSheetSet_SyncProperties() 
        { 
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Create a new sheet set file 
            AcSmDatabase sheetSetDatabase = sheetSetManager.CreateDatabase("C:\\Datasets\\CP318-4\\CP318-4.dst", "", true); 
            
            // Get the sheet set from the database 
            AcSmSheetSet sheetSet = sheetSetDatabase.GetSheetSet(); 
            
            // Attempt to lock the database 
            if (LockDatabase(ref sheetSetDatabase, true) == true) { 
                // Get the folder the sheet set is stored in 
                string sheetSetFolder = null;
                sheetSetFolder = sheetSetDatabase.GetFileName().Substring(0, sheetSetDatabase.GetFileName().LastIndexOf("\\"));
                
                // Set the default values of the sheet set 
                SetSheetSetDefaults(sheetSetDatabase, "CP318-4", "AU2009 Sheet Set Object Demo", sheetSetFolder, "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet"); 
                
                // Create a sheet set property 
                SetCustomProperty(sheetSet, "Project Approved By", "AU09", PropertyFlags.CUSTOM_SHEETSET_PROP); 
                
                // Create sheet properties 
                SetCustomProperty(sheetSet, "Checked By", "LAA", PropertyFlags.CUSTOM_SHEET_PROP);

                SetCustomProperty(sheetSet, "Complete Percentage", "0%", PropertyFlags.CUSTOM_SHEET_PROP); 
                
                AddSheet(sheetSetDatabase, "Title Page", "Project Title Page", "Title Page", "T1"); 
                
                // Create two new subsets 
                AcSmSubset subset = default(AcSmSubset); 
                subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", false); 
                
                AddSheet(subset, "North Plan", "Northern section of building plan", "North Plan", "P1"); 
                
                subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", true); 
                
                // Add a sheet property 
                SetCustomProperty(sheetSet, "Drafted By", "KMA", PropertyFlags.CUSTOM_SHEET_PROP); 
                
                // Add a subset property 
                SetCustomProperty(sheetSet, "Count", "0", PropertyFlags.CUSTOM_SHEETSET_PROP); 
                
                // Sync the properties of the sheet set with the sheets and subsets 
                SyncProperties(sheetSetDatabase); 
                
                // Unlock the database 
                LockDatabase(ref sheetSetDatabase, false); 
            } 
            else { 
                // Display error message 
                MessageBox.Show("Sheet set could not be opened for write."); 
            } 
            
            // Close the sheet set 
            sheetSetManager.Close(sheetSetDatabase); 
        } 
        
        // Register the event handlers 
        [CommandMethod("ADSK_AddSSmEvents")] 
        public void AddSSmEvents() 
        { 
            
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            try { 
                // Open a Sheet Set file 
                IAcSmDatabase sheetSetDatabase = sheetSetManager.FindOpenDatabase("C:\\Datasets\\CP318-4\\CP318-4.dst"); 
                
                // Get the sheet set from the database 
                IAcSmSheetSet sheetSet = sheetSetDatabase.GetSheetSet(); 
                
                AddEvents(sheetSetManager, sheetSetDatabase, sheetSet); 
            } 
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                MessageBox.Show(ex.Message, "Error"); 
            } 
            catch (System.Exception ex) {
                MessageBox.Show(ex.Message, "Error"); 
            } 
        } 
        
        // Remove the registered event handlers 
        [CommandMethod("ADSK_RemoveSSmEvents")] 
        public void RemoveSSmEvents() 
        { 
            RemoveEvents(); 
        } 
        
        // Counts up the sheets for all the open sheet sets 
        [CommandMethod("ADSK_SetSheetCount")] 
        public void SetSheetCount() 
        { 
            int nSheetCount = 0; 
            
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Get the loaded databases 
            IAcSmEnumDatabase enumDatabase = sheetSetManager.GetDatabaseEnumerator(); 
            
            // Get the first open database 
            IAcSmPersist item = enumDatabase.Next(); 
            
            AcSmDatabase sheetSetDatabase = default(AcSmDatabase); 
            
            while ((item != null)) {
                sheetSetDatabase = (AcSmDatabase)item; 
                
                // Attempt to lock the database 
                if (LockDatabase(ref sheetSetDatabase, true) == true) { 
                    
                    // Get the enumerator for the objects in the sheet set 
                    IAcSmEnumPersist enumerator = sheetSetDatabase.GetEnumerator();
                    IAcSmPersist itemSheetSet = enumerator.Next();  
                    
                    // Step through the objects in the sheet set 
                    while ((itemSheetSet != null)) { 
                        // Increment the counter of the object is a sheet 
                        if (itemSheetSet.GetTypeName() == "AcSmSheet") { 
                            nSheetCount = nSheetCount + 1; 
                        } 
                        
                        // Get next object 
                        itemSheetSet = enumerator.Next(); 
                    } 
                    
                    // Create a sheet set property 
                    SetCustomProperty(sheetSetDatabase.GetSheetSet(), "Total Sheets", nSheetCount.ToString(), PropertyFlags.CUSTOM_SHEETSET_PROP); 
                    
                    // Unlock the database 
                    LockDatabase(ref sheetSetDatabase, false); 
                    
                    // Clear and check for the next open sheet set 
                    nSheetCount = 0; 
                } 
                else {
                    MessageBox.Show("Unable to access " + sheetSetDatabase.GetSheetSet().GetName()); 
                } 
                
                item = enumDatabase.Next(); 
            } 
        } 
        
        // Create a new sheet set with all custom functions 
        [CommandMethod("ADSK_SheetSetFinal")] 
        public void CreateSheetSet_SheetSetFinal() 
        { 
            // Get a reference to the Sheet Set Manager object 
            IAcSmSheetSetMgr sheetSetManager = new AcSmSheetSetMgr(); 
            
            // Create a new sheet set file 
            AcSmDatabase sheetSetDatabase = sheetSetManager.CreateDatabase("C:\\Datasets\\CP318-4\\CP318-4.dst", "", true); 
            
            // Get the sheet set from the database 
            AcSmSheetSet sheetSet = sheetSetDatabase.GetSheetSet(); 
            
            // Attempt to lock the database 
            if (LockDatabase(ref sheetSetDatabase, true) == true) { 
                AcSmSheetSelSet sheetSetSel = AddSheetSelectionSet(sheetSetDatabase, "CustomSS", "Custom Selection Set"); 
                
                // Get the folder the sheet set is stored in 
                string sheetSetFolder = sheetSetDatabase.GetFileName().Substring(0, sheetSetDatabase.GetFileName().LastIndexOf("\\"));
                
                // Set the default values of the sheet set 
                SetSheetSetDefaults(sheetSetDatabase, "CP318-4", "AU2009 Sheet Set Object Demo", sheetSetFolder, "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet"); 
                
                // Create a sheet set property 
                SetCustomProperty(sheetSet, "Project Approved By", "AU09", PropertyFlags.CUSTOM_SHEETSET_PROP); 
                
                // Create sheet properties 
                SetCustomProperty(sheetSet, "Checked By", "LAA", PropertyFlags.CUSTOM_SHEET_PROP);

                SetCustomProperty(sheetSet, "Complete Percentage", "0%", PropertyFlags.CUSTOM_SHEET_PROP); 
                
                // Create a new sheet and add it to the sheet selection set 
                sheetSetSel.Add(AddSheet(sheetSetDatabase, "Title Page", "Project Title Page", "Title Page", "T1")); 
                
                // Create two new subsets 
                AcSmSubset subset = default(AcSmSubset); 
                subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", false); 
                
                // Add the subset the to the sheet selection set 
                sheetSetSel.Add(subset); 
                
                AddSheet(subset, "North Plan", "Northern section of building plan", "North Plan", "P1"); 
                
                subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", "", "C:\\Datasets\\CP318-4\\CP318-4.dwt", "Sheet", true); 
                
                // Add a sheet property 
                SetCustomProperty(sheetSet, "Drafted By", "KMA", PropertyFlags.CUSTOM_SHEET_PROP); 
                
                // Add a subset property 
                SetCustomProperty(sheetSet, "Count", "0", PropertyFlags.CUSTOM_SHEETSET_PROP); 
                
                // Sync the properties of the sheet set with the sheets and subsets 
                SyncProperties(sheetSetDatabase); 
                
                // Assign a callout and label block to the sheet set 
                AddCalloutBlock(sheetSetDatabase, "Callout Bubble", "C:\\Datasets\\CP318-4\\CP318-4.dwt"); 
                AddLabelBlock(sheetSetDatabase, "Drawing Title", "C:\\Datasets\\CP318-4\\CP318-4.dwt"); 
                
                // Assign a drawing resource location 
                AddResourceFileLocation(sheetSetDatabase, "C:\\Datasets\\CP318-4\\Res"); 
                
                // Assign a page setup override to the sheet set 
                AddPageSetupOverride(sheetSetDatabase, "DWF6_ARCH D TO B _LAND", "C:\\Datasets\\CP318-4\\CP318-4.dwt"); 
                
                // Unlock the database 
                LockDatabase(ref sheetSetDatabase, false); 
            } 
            else { 
                // Display error message 
                MessageBox.Show("Sheet set could not be opened for write."); 
            } 
            
            // Close the sheet set 
            sheetSetManager.Close(sheetSetDatabase); 
        } 
        
        // Used to lock/unlock a sheet set database 
        private bool LockDatabase(ref AcSmDatabase database, bool lockFlag) 
        { 
            bool dbLock = false; 
            // If lockFalg equals True then attempt to lock the database, otherwise 
            // attempt to unlock it. 
            if (lockFlag == true & database.GetLockStatus() == AcSmLockStatus.AcSmLockStatus_UnLocked) { 
                database.LockDb(database);
                dbLock = true; 
            } 
            else if (lockFlag == false && database.GetLockStatus() == AcSmLockStatus.AcSmLockStatus_Locked_Local) { 
                database.UnlockDb(database, true);
                dbLock = true; 
            } 
            else {
                dbLock = false; 
            }

            return dbLock; 
        } 
        
        // Used to add a subset to a sheet set 
        private AcSmSubset CreateSubset(AcSmDatabase sheetSetDatabase, string name, string description)
        {
            return CreateSubset(sheetSetDatabase, name, description, "", "", "", false);
        }

        private AcSmSubset CreateSubset(AcSmDatabase sheetSetDatabase, string name, string description, string newSheetLocation)
        {
            return CreateSubset(sheetSetDatabase, name, description, newSheetLocation, "", "", false);
        }

        private AcSmSubset CreateSubset(AcSmDatabase sheetSetDatabase, string name, string description, string newSheetLocation, string newSheetDWTLocation)
        {
            return CreateSubset(sheetSetDatabase, name, description, newSheetLocation, newSheetDWTLocation, "", false);
        }

        private AcSmSubset CreateSubset(AcSmDatabase sheetSetDatabase, string name, string description, string newSheetLocation, string newSheetDWTLocation, string newSheetDWTLayout)
        {
            return CreateSubset(sheetSetDatabase, name, description, newSheetLocation, newSheetDWTLocation, newSheetDWTLayout, false);
        }

        private AcSmSubset CreateSubset(AcSmDatabase sheetSetDatabase, string name, string description, string newSheetLocation,
                                        string newSheetDWTLocation, string newSheetDWTLayout, bool promptForDWT)
        { 
            // Create a subset with the provided name and description 
            AcSmSubset subset = (AcSmSubset)sheetSetDatabase.GetSheetSet().CreateSubset(name, description); 
            
            // Get the folder the sheet set is stored in 
            string sheetSetFolder = sheetSetDatabase.GetFileName().Substring(0, sheetSetDatabase.GetFileName().LastIndexOf("\\"));
            
            // Create a reference to a File Reference object 
            IAcSmFileReference fileReference = subset.GetNewSheetLocation(); 
            
            // Check to see if a path was provided, if not default 
            // to the location of the sheet set 
            if (!string.IsNullOrEmpty(newSheetLocation)) { 
                fileReference.SetFileName(newSheetLocation); 
            } 
            else { 
                fileReference.SetFileName(sheetSetFolder); 
            } 
            
            // Set the location for new sheets added to the subset 
            subset.SetNewSheetLocation(fileReference); 
            
            // Create a reference to a Layout Reference object 
            AcSmAcDbLayoutReference layoutReference = default(AcSmAcDbLayoutReference);
            layoutReference = subset.GetDefDwtLayout(); 
            
            // Check to see that a default DWT location and name was provided 
            if (!string.IsNullOrEmpty(newSheetDWTLocation)) { 
                // Set the template location and name of the layout 
                //for the Layout Reference object 
                layoutReference.SetFileName(newSheetDWTLocation); 
                layoutReference.SetName(newSheetDWTLayout); 
                
                // Set the Layout Reference for the subset 
                subset.SetDefDwtLayout(layoutReference); 
            } 
            
            // Set the Prompt for Template option of the subset 
            subset.SetPromptForDwt(promptForDWT);
            return subset; 
        } 
        
        // Used to add a sheet to a sheet set or subset 
        // Note: This function is dependent on a Default Template and Storage location 
        // being set for the sheet set or subset. 
        private AcSmSheet AddSheet(IAcSmComponent component, string name, string description, string title, string number) 
        { 
            AcSmSheet sheet = default(AcSmSheet); 
            // Check to see if the component is a sheet set or subset, 
            // and create the new sheet based on the component's type 
            if (component.GetTypeName() == "AcSmSubset") { 
                AcSmSubset subset = (AcSmSubset)component;
                sheet = subset.AddNewSheet(name, description); 
                
                // Add the sheet as the first one in the subset 
                subset.InsertComponent(sheet, null); 
            } 
            else {
                sheet = component.GetDatabase().GetSheetSet().AddNewSheet(name, description); 
                
                // Add the sheet as the first one in the sheet set
                component.GetDatabase().GetSheetSet().InsertComponent(sheet, null); 
            } 
            
            // Set the number and title of the sheet 
            sheet.SetNumber(number);
            sheet.SetTitle(title);
            return sheet; 
        } 

        // Set the default properties of a sheet set
        private void SetSheetSetDefaults(AcSmDatabase sheetSetDatabase, string name)
        {
            SetSheetSetDefaults(sheetSetDatabase, name, "", "", "", "", false);
        }
        
        private void SetSheetSetDefaults(AcSmDatabase sheetSetDatabase, string name, string description)
        {
            SetSheetSetDefaults(sheetSetDatabase, name, description, "", "", "", false);
        }

        private void SetSheetSetDefaults(AcSmDatabase sheetSetDatabase, string name, string description,
                                         string newSheetLocation)
        {
            SetSheetSetDefaults(sheetSetDatabase, name, description, newSheetLocation, "", "", false);
        }

        private void SetSheetSetDefaults(AcSmDatabase sheetSetDatabase, string name, string description,
                                         string newSheetLocation, string newSheetDWTLocation)
        {
            SetSheetSetDefaults(sheetSetDatabase, name, description, newSheetLocation, newSheetDWTLocation, "", false);
        }

        private void SetSheetSetDefaults(AcSmDatabase sheetSetDatabase, string name, string description,
                                         string newSheetLocation, string newSheetDWTLocation, string newSheetDWTLayout)
        {
            SetSheetSetDefaults(sheetSetDatabase, name, description, newSheetLocation, newSheetDWTLocation, newSheetDWTLayout, false);
        }

        private void SetSheetSetDefaults(AcSmDatabase sheetSetDatabase, string name, string description,
                                         string newSheetLocation, string newSheetDWTLocation, string newSheetDWTLayout,
                                         bool promptForDWT) 
        {
            // Set the Name and Description for the sheet set 
            sheetSetDatabase.GetSheetSet().SetName(name); 
            sheetSetDatabase.GetSheetSet().SetDesc(description); 
            
            // Check to see if a Storage Location was provided 
            if (!string.IsNullOrEmpty(newSheetLocation)) { 
                // Get the folder the sheet set is stored in 
                string sheetSetFolder = sheetSetDatabase.GetFileName().Substring(0, sheetSetDatabase.GetFileName().LastIndexOf("\\"));
                
                // Create a reference to a File Reference object 
                IAcSmFileReference fileReference = default(IAcSmFileReference); 
                fileReference = sheetSetDatabase.GetSheetSet().GetNewSheetLocation(); 
                
                // Set the default storage location based on the location of the sheet set 
                fileReference.SetFileName(sheetSetFolder); 
                
                // Set the new Sheet location for the sheet set 
                sheetSetDatabase.GetSheetSet().SetNewSheetLocation(fileReference); 
            } 
            
            // Check to see if a Template was provided 
            if (!string.IsNullOrEmpty(newSheetDWTLocation)) { 
                // Set the Default Template for the sheet set 
                AcSmAcDbLayoutReference layoutReference = default(AcSmAcDbLayoutReference); 
                layoutReference = sheetSetDatabase.GetSheetSet().GetDefDwtLayout(); 
                
                // Set the template location and name of the layout 
                // for the Layout Reference object 
                layoutReference.SetFileName(newSheetDWTLocation); 
                layoutReference.SetName(newSheetDWTLayout); 
                
                // Set the Layout Reference for the sheet set 
                sheetSetDatabase.GetSheetSet().SetDefDwtLayout(layoutReference); 
            } 
            
            // Set the Prompt for Template option of the subset 
            sheetSetDatabase.GetSheetSet().SetPromptForDwt(promptForDWT); 
        } 
        
        // Import a sheet into a sheet set or subset 
        private AcSmSheet ImportASheet(IAcSmComponent component, string title, string description, string number, string fileName, string layout) 
        { 
            
            AcSmSheet sheet = default(AcSmSheet); 
            // Create a reference to a Layout Reference object 
            AcSmAcDbLayoutReference layoutReference = new AcSmAcDbLayoutReference(); 
            layoutReference.InitNew(component); 
            
            // Set the layout and drawing file to use for the sheet 
            layoutReference.SetFileName(fileName); 
            layoutReference.SetName(layout); 
            
            // Import the sheet into the sheet set 
            // Check to see if the Component is a Subset or Sheet Set 
            if (component.GetTypeName() == "AcSmSubset") { 
                AcSmSubset subset = (AcSmSubset)component;

                sheet = subset.ImportSheet(layoutReference);
                subset.InsertComponent(sheet, null); 
            } 
            else { 
                AcSmDatabase sheetSetDatabase = (AcSmDatabase)component;

                sheet = sheetSetDatabase.GetSheetSet().ImportSheet(layoutReference);
                sheetSetDatabase.GetSheetSet().InsertComponent(sheet, null); 
            } 
            
            // Set the properties of the sheet 
            sheet.SetDesc(description);
            sheet.SetTitle(title);
            sheet.SetNumber(number);
            return sheet; 
        } 
        
        // Set/create a custom sheet or sheet set property 
        private void SetCustomProperty(IAcSmPersist owner, string propertyName, object propertyValue, PropertyFlags sheetSetFlag) 
        { 
            // Create a reference to the Custom Property Bag
            AcSmCustomPropertyBag customPropertyBag = default(AcSmCustomPropertyBag);

            if (owner.GetTypeName() == "AcSmSheet")
            {
                AcSmSheet sheet = (AcSmSheet)owner;
                customPropertyBag = sheet.GetCustomPropertyBag();
            }
            else
            {
                AcSmSheetSet sheetSet = (AcSmSheetSet)owner;
                customPropertyBag = sheetSet.GetCustomPropertyBag();
            }

            // Create a reference to a Custom Property Value 
            AcSmCustomPropertyValue customPropertyValue = new AcSmCustomPropertyValue();
            customPropertyValue.InitNew(owner); 
            
            // Set the flag for the property 
            customPropertyValue.SetFlags(sheetSetFlag); 
            
            // Set the value for the property 
            customPropertyValue.SetValue(propertyValue); 
            
            // Create the property 
            customPropertyBag.SetProperty(propertyName, customPropertyValue); 
        } 
        
        // Add a callout block to the sheet set 
        public void AddCalloutBlock(AcSmDatabase sheetSetDatabase, string blockName, string drawingFile) 
        { 
            
            IAcSmCalloutBlocks calloutBlocks = default(IAcSmCalloutBlocks); 
            AcSmAcDbBlockRecordReference calloutBlockReference = new AcSmAcDbBlockRecordReference(); 
            
            // Get the callout blocks for the sheet set 
            calloutBlocks = sheetSetDatabase.GetSheetSet().GetCalloutBlocks(); 
            
            // Create a new reference to a callout block and define the block to use 
            calloutBlockReference.InitNew(sheetSetDatabase); 
            calloutBlockReference.SetFileName(drawingFile); 
            calloutBlockReference.SetName(blockName); 
            
            // Add the new block to the callout blocks for the sheet set 
            calloutBlocks.Add(calloutBlockReference); 
        } 
        
        // Add a label block to the sheet set 
        public void AddLabelBlock(AcSmDatabase sheetSetDatabase, string blockName, string drawingFile) 
        { 
            
            AcSmAcDbBlockRecordReference labelBlockReference = new AcSmAcDbBlockRecordReference(); 
            
            // Create a reference to a layout label block 
            labelBlockReference.InitNew(sheetSetDatabase); 
            labelBlockReference.SetFileName(drawingFile); 
            labelBlockReference.SetName(blockName); 
            
            // Set the label block for the sheet set 
            sheetSetDatabase.GetSheetSet().SetDefLabelBlk(labelBlockReference); 
        } 
        
        // Add a drawing resource location to the sheet set 
        public void AddResourceFileLocation(AcSmDatabase sheetSetDatabase, string resourceLoc) 
        { 
            IAcSmResources resources = default(IAcSmResources); 
            AcSmFileReference fileReference = new AcSmFileReference(); 
            
            // Create a reference to a resource location 
            resources = sheetSetDatabase.GetSheetSet().GetResources(); 
            fileReference.InitNew(sheetSetDatabase); 
            fileReference.SetFileName(resourceLoc); 
            
            // Add the drawing resource location 
            resources.Add(fileReference); 
        } 
        
        // Add a page setup override to the sheet set 
        public void AddPageSetupOverride(AcSmDatabase sheetSetDatabase, string pageSetupName, string drawingFile) 
        { 
            
            AcSmFileReference fileReference = new AcSmFileReference(); 
            
            // Create a new reference to the page setup overrides to use 
            fileReference.InitNew(sheetSetDatabase); 
            fileReference.SetFileName(drawingFile); 
            
            // Set the drawing template that contains the page setup overrides 
            sheetSetDatabase.GetSheetSet().SetAltPageSetups(fileReference); 
        } 
        
        // Add a sheet selection set 
        public AcSmSheetSelSet AddSheetSelectionSet(AcSmDatabase sheetSetDatabase, string name, string description) 
        { 
            
            // Get the sheet selection sets for the sheet set 
            AcSmSheetSelSets sheetSelSets = default(AcSmSheetSelSets); 
            sheetSelSets = sheetSetDatabase.GetSheetSet().GetSheetSelSets(); 
            
            // Add the new sheet selection set 
            AcSmSheetSelSet sheetSelSet = null; 
            sheetSelSets.Add(name, description, ref sheetSelSet); 
            
            // Return the selection set added 
            return sheetSelSet; 
        } 
        
        // Synchronize the properties of a sheet with the sheet set 
        private void SyncProperties(AcSmDatabase sheetSetDatabase) 
        { 
            // Get the objects in the sheet set 
            IAcSmEnumPersist enumerator = sheetSetDatabase.GetEnumerator(); 
            
            // Get the first object in the Enumerator 
            IAcSmPersist item = enumerator.Next(); 
            
            // Step through all the objects in the sheet set 
            while ((item != null)) { 
                IAcSmSheet sheet = null; 
                
                // Check to see if the object is a sheet 
                if (item.GetTypeName() == "AcSmSheet") { 
                    sheet = (IAcSmSheet)item; 
                    
                    // Create a reference to the Property Enumerator for 
                    // the Custom Property Bag 
                    IAcSmEnumProperty enumeratorProperty = item.GetDatabase().GetSheetSet().GetCustomPropertyBag().GetPropertyEnumerator(); 
                    
                    // Get the values from the Sheet Set to populate to the sheets 
                    string name = ""; 
                    AcSmCustomPropertyValue customPropertyValue = null; 
                    
                    // Get the first property 
                    enumeratorProperty.Next(out name, out customPropertyValue); 
                    
                    // Step through each of the properties 
                    while ((customPropertyValue != null)) { 
                        // Check to see if the property is for a sheet 
                        if (customPropertyValue.GetFlags() == PropertyFlags.CUSTOM_SHEET_PROP) { 
                            
                            //// Create a reference to the Custom Property Bag 
                            //AcSmCustomPropertyBag customSheetPropertyBag = sheet.GetCustomPropertyBag(); 
                            
                            //// Create a reference to a Custom Property Value 
                            //AcSmCustomPropertyValue customSheetPropertyValue = new AcSmCustomPropertyValue();
                            //customSheetPropertyValue.InitNew(sheet); 
                            
                            //// Set the flag for the property 
                            //customSheetPropertyValue.SetFlags(customPropertyValue.GetFlags()); 
                            
                            //// Set the value for the property 
                            //customSheetPropertyValue.SetValue(customPropertyValue.GetValue()); 
                            
                            //// Create the property 
                            //customSheetPropertyBag.SetProperty(name, customSheetPropertyValue); 

                            SetCustomProperty(sheet, name, customPropertyValue.GetValue(), customPropertyValue.GetFlags()); 
                        } 
                        
                        // Get the next property 
                        enumeratorProperty.Next(out name, out customPropertyValue); 
                    } 
                } 
                
                // Get the next Sheet 
                item = enumerator.Next(); 
            } 
        } 
        
        // Register event handlers with the Sheet Set Manager, 
        // database, and sheet set 
        private void AddEvents(IAcSmSheetSetMgr sheetSetManager, IAcSmDatabase sheetSetDatabase, IAcSmSheetSet sheetSet) 
        { 
            // ERROR: Not supported in C#: OnErrorStatement 
            if ((eventHandler != null)) { 
                return; 
            } 
            
            eventHandler = new MySSmEventHandler(); 
            
            m_sheetSetManager = sheetSetManager; 
            m_sheetSetDatabase = sheetSetDatabase; 
            m_sheetSet = sheetSet; 
            
            // Register an event for the Sheet Set Manager 
            eventSSMCookie = m_sheetSetManager.Register(eventHandler); 
            
            // Register a database event 
            eventDbCookie = m_sheetSetDatabase.Register(eventHandler); 
            
            // Register for sheet set event 
            eventSSetCookie = m_sheetSet.Register(eventHandler); 
        } 
        
        // Remove the registered event handlers for the Sheet Set Manager, 
        // database, and sheet set 
        private void RemoveEvents() 
        { 
            // ERROR: Not supported in C#: OnErrorStatement 
            if (eventHandler == null) { 
                return; 
            } 
            
            m_sheetSetManager.Unregister(eventSSMCookie); 
            m_sheetSetDatabase.Unregister(eventDbCookie); 
            m_sheetSet.Unregister(eventSSetCookie); 
            
            eventHandler = null; 
        }
    }

    // Custom event handler class 
    public class MySSmEventHandler : IAcSmEvents
    {
        private void IAcSmEvents_OnChanged(AcSmEvent ev, IAcSmPersist comp)
        {
            Autodesk.AutoCAD.ApplicationServices.Document activeDocument = default(Autodesk.AutoCAD.ApplicationServices.Document);
            activeDocument = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;

            AcSmSheet oSheet = default(AcSmSheet);
            AcSmSubset oSubset = default(AcSmSubset);

            if (ev == AcSmEvent.ACSM_DATABASE_OPENED)
            {
                activeDocument.Editor.WriteMessage("\n" + comp.GetDatabase().GetFileName() + " was opened.");
            }
            if (ev == AcSmEvent.ACSM_DATABASE_CHANGED)
            {
                activeDocument.Editor.WriteMessage("\n" + comp.GetDatabase().GetFileName() + " database changed.");
            }
            if (ev == AcSmEvent.SHEET_DELETED)
            {
                oSheet = (AcSmSheet)comp;
                activeDocument.Editor.WriteMessage("\n" + oSheet.GetName() + " was deleted.");
            }
            if (ev == AcSmEvent.SHEET_SUBSET_CREATED)
            {
                oSubset = (AcSmSubset)comp;
                activeDocument.Editor.WriteMessage("\n" + oSubset.GetName() + " was created.");
            }
            if (ev == AcSmEvent.SHEET_SUBSET_DELETED)
            {
                oSubset = (AcSmSubset)comp;
                activeDocument.Editor.WriteMessage("\n" + oSubset.GetName() + " was deleted.");
            }
        }
        void IAcSmEvents.OnChanged(AcSmEvent ev, IAcSmPersist comp)
        {
            IAcSmEvents_OnChanged(ev, comp);
        }
    }
}