﻿Imports Autodesk.AutoCAD.Runtime
Imports ACSMCOMPONENTS18Lib

Public Class ADSK_SheetSet
    Dim eventHandler As MySSmEventHandler
    Dim eventSSMCookie As Long
    Dim eventDbCookie As Long
    Dim eventSSetCookie As Long
    Dim m_sheetSetManager As IAcSmSheetSetMgr
    Dim m_sheetSetDatabase As IAcSmDatabase
    Dim m_sheetSet As IAcSmSheetSet

    ' Open a Sheet Set
    <CommandMethod("ADSK_OpenSheetSet")> _
    Public Sub OpenSheetSet()
        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Open a Sheet Set file
        Dim sheetSetDatabase As AcSmDatabase
        sheetSetDatabase = sheetSetManager.OpenDatabase("C:\Program Files\AutoCAD 2010\Sample\Sheet Sets\Architectural\IRD Addition.dst", False)

        ' Return the namd and description of the sheet set
        MsgBox("Sheet Set Name: " + sheetSetDatabase.GetSheetSet().GetName() & vbCrLf + _
               "Sheet Set Description: " + sheetSetDatabase.GetSheetSet().GetDesc())

        ' Close the sheet set
        sheetSetManager.Close(sheetSetDatabase)
    End Sub

    ' Create a new sheet set
    <CommandMethod("ADSK_CreateSheetSet")> _
    Public Sub CreateSheetSet()
        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Create a new sheet set file
        Dim sheetSetDatabase As AcSmDatabase
        sheetSetDatabase = sheetSetManager.CreateDatabase("C:\Datasets\CP318-4\CP318-4.dst", _
                                                          "", True)

        ' Get the sheet set from the database
        Dim sheetSet As AcSmSheetSet
        sheetSet = sheetSetDatabase.GetSheetSet()

        ' Set the name and description of the sheet set
        sheetSet.SetName("CP318-4")
        sheetSet.SetDesc("AU2009 Sheet Set Object Demo")

        ' Return the name and description of the sheet set
        MsgBox("Sheet Set Name: " + sheetSetDatabase.GetSheetSet().GetName() + vbCrLf + _
               "Sheet Set Description: " + sheetSetDatabase.GetSheetSet().GetDesc())

        ' Close the sheet set
        sheetSetManager.Close(sheetSetDatabase)
    End Sub

    ' Create a new sheet set
    <CommandMethod("ADSK_CreateSheetSet_LockExample")> _
    Public Sub CreateSheetSet_LockExample()
        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Create a new sheet set file
        Dim sheetSetDatabase As AcSmDatabase
        sheetSetDatabase = sheetSetManager.CreateDatabase("C:\Datasets\CP318-4\CP318-4.dst", _
                                                          "", True)

        ' Get the sheet set from the database
        Dim sheetSet As AcSmSheetSet
        sheetSet = sheetSetDatabase.GetSheetSet()

        ' Attempt to lock the database
        If LockDatabase(sheetSetDatabase, True) = True Then
            ' Set the name and description of the sheet set
            sheetSet.SetName("CP318-4")
            sheetSet.SetDesc("AU2009 Sheet Set Object Demo")

            ' Unlock the database
            LockDatabase(sheetSetDatabase, False)

            ' Return the name and description of the sheet set
            MsgBox("Sheet Set Name: " + sheetSetDatabase.GetSheetSet().GetName() & vbCrLf + _
                   "Sheet Set Description: " + sheetSetDatabase.GetSheetSet().GetDesc())
        Else
            ' Display error message
            MsgBox("Sheet set could not be opened for write.")
        End If

        ' Close the sheet set
        sheetSetManager.Close(sheetSetDatabase)
    End Sub

    ' Step through all open sheet sets
    <CommandMethod("ADSK_StepThroughTheOpenSheetSets")> _
    Public Sub StepThroughTheOpenSheetSets()
        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Get the loaded databases
        Dim enumDatabase As IAcSmEnumDatabase
        enumDatabase = sheetSetManager.GetDatabaseEnumerator()

        ' Get the first open database
        Dim item As IAcSmPersist
        item = enumDatabase.Next()

        Dim customMessage As String = ""

        ' If a database is open continue
        If Not item Is Nothing Then
            Dim count As Integer = 0

            ' Step through the database enumerator
            Do While Not item Is Nothing
                ' Append the file name of the open sheet set to the output string
                customMessage = customMessage + vbLf + _
                                item.GetDatabase().GetFileName()

                ' Get the next open database and increment the counter
                item = enumDatabase.Next()
                count = count + 1
            Loop

            customMessage = "Sheet sets open: " + count.ToString() + _
                            customMessage
        Else
            customMessage = "No sheet sets are currently open."
        End If

        ' Display the custom message
        MsgBox(customMessage)
    End Sub

    ' Create a new sheet set
    <CommandMethod("ADSK_CreateSheetSet_AddSubset")> _
    Public Sub CreateSheetSet_AddSubset()
        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Create a new sheet set file
        Dim sheetSetDatabase As AcSmDatabase
        sheetSetDatabase = sheetSetManager.CreateDatabase("C:\Datasets\CP318-4\CP318-4.dst", _
                                                          "", True)

        ' Get the sheet set from the database
        Dim sheetSet As AcSmSheetSet
        sheetSet = sheetSetDatabase.GetSheetSet()

        ' Attempt to lock the database
        If LockDatabase(sheetSetDatabase, True) = True Then
            ' Set the name and description of the sheet set
            sheetSet.SetName("CP318-4")
            sheetSet.SetDesc("AU2009 Sheet Set Object Demo")

            ' Create two new subsets
            Dim subset As AcSmSubset
            subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", _
                                  "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", False)

            subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", "", _
                                  "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", True)

            ' Unlock the database
            LockDatabase(sheetSetDatabase, False)
        Else
            ' Display error message
            MsgBox("Sheet set could not be opened for write.")
        End If

        ' Close the sheet set
        sheetSetManager.Close(sheetSetDatabase)
    End Sub

    ' Create a new sheet set
    <CommandMethod("ADSK_CreateSheetSet_AddSheet")> _
    Public Sub CreateSheetSet_AddSheet()
        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Create a new sheet set file
        Dim sheetSetDatabase As AcSmDatabase
        sheetSetDatabase = sheetSetManager.CreateDatabase("C:\Datasets\CP318-4\CP318-4.dst", _
                                                          "", True)

        ' Get the sheet set from the database
        Dim sheetSet As AcSmSheetSet
        sheetSet = sheetSetDatabase.GetSheetSet()

        ' Attempt to lock the database
        If LockDatabase(sheetSetDatabase, True) = True Then
            ' Get the folder the sheet set is stored in
            Dim sheetSetFolder As String
            sheetSetFolder = Mid(sheetSetDatabase.GetFileName(), 1, _
                             InStrRev(sheetSetDatabase.GetFileName(), "\"))

            ' set the default values of the sheet set
            SetSheetSetDefaults(sheetSetDatabase, _
                                "CP318-4", "AU2009 Sheet Set Object Demo", _
                                sheetSetFolder, _
                                "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet")

            AddSheet(sheetSetDatabase, "Title Page", "Project Title Page", _
                     "Title Page", "T1")

            ' Create two new subsets
            Dim subset As AcSmSubset
            subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", _
                                  "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", False)

            AddSheet(subset, "North Plan", "Northern section of building plan", _
                     "North Plan", "P1")

            subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", _
                                  "", "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", True)

            ' Unlock the database
            LockDatabase(sheetSetDatabase, False)
        Else
            ' Display error message
            MsgBox("Sheet set could not be opened for write.")
        End If

        ' Close the sheet set
        sheetSetManager.Close(sheetSetDatabase)
    End Sub

    ' Create a new sheet set
    <CommandMethod("ADSK_CreateSheetSet_AddCustomProperty")> _
    Public Sub CreateSheetSet_AddCustomProperty()
        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Create a new sheet set file
        Dim sheetSetDatabase As AcSmDatabase
        sheetSetDatabase = sheetSetManager.CreateDatabase("C:\Datasets\CP318-4\CP318-4.dst", _
                                                          "", True)

        ' Get the sheet set from the database
        Dim sheetSet As AcSmSheetSet
        sheetSet = sheetSetDatabase.GetSheetSet()

        ' Attempt to lock the database
        If LockDatabase(sheetSetDatabase, True) = True Then
            ' Get the folder the sheet set is stored in
            Dim sheetSetFolder As String
            sheetSetFolder = Mid(sheetSetDatabase.GetFileName(), 1, _
                             InStrRev(sheetSetDatabase.GetFileName(), "\"))

            ' Set the default values of the sheet set
            SetSheetSetDefaults(sheetSetDatabase, _
                                "CP318-4", "AU2009 Sheet Set Object Demo", _
                                sheetSetFolder, _
                                "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet")

            ' Create a sheet set property
            SetCustomProperty(sheetSet, "Project Approved By", "AU09", _
                              PropertyFlags.CUSTOM_SHEETSET_PROP)

            ' Create sheet properties
            SetCustomProperty(sheetSet, "Checked By", "LAA", _
                              PropertyFlags.CUSTOM_SHEET_PROP)

            SetCustomProperty(sheetSet, "Complete Percentage", "0%", _
                              PropertyFlags.CUSTOM_SHEET_PROP)

            AddSheet(sheetSetDatabase, "Title Page", "Project Title Page", _
                     "Title Page", "T1")

            ' Create two new subsets
            Dim subset As AcSmSubset
            subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", _
                                  "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", False)

            AddSheet(subset, "North Plan", "Northern section of building plan", _
                     "North Plan", "P1")

            subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", _
                                  "", "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", True)

            ' Unlock the database
            LockDatabase(sheetSetDatabase, False)
        Else
            ' Display error message
            MsgBox("Sheet set could not be opened for write.")
        End If

        ' Close the sheet set
        sheetSetManager.Close(sheetSetDatabase)
    End Sub

    ' Create a new sheet set
    <CommandMethod("ADSK_CreateSheetSet_SyncProperties")> _
    Public Sub CreateSheetSet_SyncProperties()
        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Create a new sheet set file
        Dim sheetSetDatabase As AcSmDatabase
        sheetSetDatabase = sheetSetManager.CreateDatabase("C:\Datasets\CP318-4\CP318-4.dst", _
                                                          "", True)

        ' Get the sheet set from the database
        Dim sheetSet As AcSmSheetSet
        sheetSet = sheetSetDatabase.GetSheetSet()

        ' Attempt to lock the database
        If LockDatabase(sheetSetDatabase, True) = True Then
            ' Get the folder the sheet set is stored in
            Dim sheetSetFolder As String
            sheetSetFolder = Mid(sheetSetDatabase.GetFileName(), 1, _
                             InStrRev(sheetSetDatabase.GetFileName(), "\"))

            ' Set the default values of the sheet set
            SetSheetSetDefaults(sheetSetDatabase, _
                                "CP318-4", "AU2009 Sheet Set Object Demo", _
                                sheetSetFolder, _
                                "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet")

            ' Create a sheet set property
            SetCustomProperty(sheetSet, "Project Approved By", "AU09", _
                              PropertyFlags.CUSTOM_SHEETSET_PROP)

            ' Create sheet properties
            SetCustomProperty(sheetSet, "Checked By", "LAA", _
                              PropertyFlags.CUSTOM_SHEET_PROP)

            SetCustomProperty(sheetSet, "Complete Percentage", "0%", _
                              PropertyFlags.CUSTOM_SHEET_PROP)

            AddSheet(sheetSetDatabase, "Title Page", "Project Title Page", _
                     "Title Page", "T1")

            ' Create two new subsets
            Dim subset As AcSmSubset
            subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", _
                                  "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", False)

            AddSheet(subset, "North Plan", "Northern section of building plan", _
                     "North Plan", "P1")

            subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", _
                                  "", "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", True)

            ' Add a sheet property
            SetCustomProperty(sheetSet, "Drafted By", "KMA", _
                              PropertyFlags.CUSTOM_SHEET_PROP)

            ' Add a subset property
            SetCustomProperty(sheetSet, "Count", "0", _
                              PropertyFlags.CUSTOM_SHEETSET_PROP)

            ' Sync the properties of the sheet set with the sheets and subsets
            SyncProperties(sheetSetDatabase)

            ' Unlock the database
            LockDatabase(sheetSetDatabase, False)
        Else
            ' Display error message
            MsgBox("Sheet set could not be opened for write.")
        End If

        ' Close the sheet set
        sheetSetManager.Close(sheetSetDatabase)
    End Sub

    ' Register the event handlers
    <CommandMethod("ADSK_AddSSmEvents")> _
    Public Sub AddSSmEvents()

        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        Try
            ' Open a Sheet Set file
            Dim sheetSetDatabase As IAcSmDatabase
            sheetSetDatabase = sheetSetManager. _
                               FindOpenDatabase("C:\Datasets\CP318-4\CP318-4.dst")

            ' Get the sheet set from the database
            Dim sheetSet As IAcSmSheetSet
            sheetSet = sheetSetDatabase.GetSheetSet()

            AddEvents(sheetSetManager, sheetSetDatabase, sheetSet)
        Catch ex As Autodesk.AutoCAD.Runtime.Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        Catch ex As System.Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    ' Remove the registered event handlers
    <CommandMethod("ADSK_RemoveSSmEvents")> _
    Public Sub RemoveSSmEvents()
        RemoveEvents()
    End Sub

    ' Counts up the sheets for all the open sheet sets
    <CommandMethod("ADSK_SetSheetCount")> _
    Public Sub SetSheetCount()
        Dim nSheetCount As Integer = 0

        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Get the loaded databases
        Dim enumDatabase As IAcSmEnumDatabase
        enumDatabase = sheetSetManager.GetDatabaseEnumerator()

        ' Get the first open database
        Dim item As IAcSmPersist
        item = enumDatabase.Next()

        Dim sheetSetDatabase As AcSmDatabase

        Do While Not item Is Nothing
            sheetSetDatabase = item

            ' Attempt to lock the database
            If LockDatabase(sheetSetDatabase, True) = True Then

                On Error Resume Next

                Dim enumerator As IAcSmEnumPersist
                Dim itemSheetSet As IAcSmPersist

                ' Get the enumerator for the objects in the sheet set
                enumerator = sheetSetDatabase.GetEnumerator()
                itemSheetSet = enumerator.Next()

                ' Step through the objects in the sheet set
                Do While Not itemSheetSet Is Nothing
                    ' Increment the counter of the object is a sheet
                    If itemSheetSet.GetTypeName() = "AcSmSheet" Then
                        nSheetCount = nSheetCount + 1
                    End If

                    ' Get next object
                    itemSheetSet = enumerator.Next()
                Loop

                ' Create a sheet set property
                SetCustomProperty(sheetSetDatabase.GetSheetSet(), "Total Sheets", CStr(nSheetCount), _
                                  PropertyFlags.CUSTOM_SHEETSET_PROP)

                ' Unlock the database
                LockDatabase(sheetSetDatabase, False)

                ' Clear and check for the next open sheet set
                nSheetCount = 0
            Else
                MsgBox("Unable to access " & sheetSetDatabase.GetSheetSet().GetName())
            End If

            item = enumDatabase.Next
        Loop
    End Sub

    ' Create a new sheet set with all custom functions
    <CommandMethod("ADSK_SheetSetFinal")> _
    Public Sub CreateSheetSet_SheetSetFinal()
        ' Get a reference to the Sheet Set Manager object
        Dim sheetSetManager As IAcSmSheetSetMgr
        sheetSetManager = New AcSmSheetSetMgr

        ' Create a new sheet set file
        Dim sheetSetDatabase As AcSmDatabase
        sheetSetDatabase = sheetSetManager.CreateDatabase("C:\Datasets\CP318-4\CP318-4.dst", "", True)

        ' Get the sheet set from the database
        Dim sheetSet As AcSmSheetSet
        sheetSet = sheetSetDatabase.GetSheetSet()

        ' Attempt to lock the database
        If LockDatabase(sheetSetDatabase, True) = True Then
            Dim sheetSetSel As AcSmSheetSelSet = AddSheetSelectionSet(sheetSetDatabase, _
                                                                      "CustomSS", _
                                                                      "Custom Selection Set")

            ' Get the folder the sheet set is stored in
            Dim sheetSetFolder As String
            sheetSetFolder = Mid(sheetSetDatabase.GetFileName(), 1, _
                             InStrRev(sheetSetDatabase.GetFileName(), "\"))

            ' Set the default values of the sheet set
            SetSheetSetDefaults(sheetSetDatabase, _
                                "CP318-4", "AU2009 Sheet Set Object Demo", _
                                sheetSetFolder, _
                                "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet")

            ' Create a sheet set property
            SetCustomProperty(sheetSet, "Project Approved By", "AU09", _
                              PropertyFlags.CUSTOM_SHEETSET_PROP)

            ' Create sheet properties
            SetCustomProperty(sheetSet, "Checked By", "LAA", _
                              PropertyFlags.CUSTOM_SHEET_PROP)

            SetCustomProperty(sheetSet, "Complete Percentage", "0%", _
                              PropertyFlags.CUSTOM_SHEET_PROP)

            ' Create a new sheet and add it to the sheet selection set
            sheetSetSel.Add(AddSheet(sheetSetDatabase, "Title Page", _
                                     "Project Title Page", "Title Page", "T1"))

            ' Create two new subsets
            Dim subset As AcSmSubset
            subset = CreateSubset(sheetSetDatabase, "Plans", "Building Plans", "", _
                                  "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", False)

            ' Add the subset the to the sheet selection set
            sheetSetSel.Add(subset)

            AddSheet(subset, "North Plan", "Northern section of building plan", _
                     "North Plan", "P1")

            subset = CreateSubset(sheetSetDatabase, "Elevations", "Building Elevations", _
                                  "", "C:\Datasets\CP318-4\CP318-4.dwt", "Sheet", True)

            ' Add a sheet property
            SetCustomProperty(sheetSet, "Drafted By", "KMA", _
                              PropertyFlags.CUSTOM_SHEET_PROP)

            ' Add a subset property
            SetCustomProperty(sheetSet, "Count", "0", _
                              PropertyFlags.CUSTOM_SHEETSET_PROP)

            ' Sync the properties of the sheet set with the sheets and subsets
            SyncProperties(sheetSetDatabase)

            ' Assign a callout and label block to the sheet set
            AddCalloutBlock(sheetSetDatabase, "Callout Bubble", _
                            "C:\Datasets\CP318-4\CP318-4.dwt")
            AddLabelBlock(sheetSetDatabase, "Drawing Title", _
                          "C:\Datasets\CP318-4\CP318-4.dwt")

            ' Assign a drawing resource location
            AddResourceFileLocation(sheetSetDatabase, "C:\Datasets\CP318-4\Res")

            ' Assign a page setup override to the sheet set
            AddPageSetupOverride(sheetSetDatabase, "DWF6_ARCH D TO B _LAND", _
                                 "C:\Datasets\CP318-4\CP318-4.dwt")

            ' Unlock the database
            LockDatabase(sheetSetDatabase, False)
        Else
            ' Display error message
            MsgBox("Sheet set could not be opened for write.")
        End If

        ' Close the sheet set
        sheetSetManager.Close(sheetSetDatabase)
    End Sub

    ' Used to lock/unlock a sheet set database
    Private Function LockDatabase(ByRef database As AcSmDatabase, _
                                  ByVal lockFlag As Boolean) As Boolean
        Dim dbLock As Boolean = False

        ' If lockFalg equals True then attempt to lock the database, otherwise
        ' attempt to unlock it.
        If lockFlag = True And _
           database.GetLockStatus() = AcSmLockStatus.AcSmLockStatus_UnLocked Then
            database.LockDb(database)
            dbLock = True
        ElseIf lockFlag = False And _
            database.GetLockStatus = AcSmLockStatus.AcSmLockStatus_Locked_Local Then
            database.UnlockDb(database)
            dbLock = True
        Else
            dbLock = False
        End If

        LockDatabase = dbLock
    End Function

    ' Used to add a subset to a sheet set
    Private Function CreateSubset(ByVal sheetSetDatabase As AcSmDatabase, _
                                  ByVal name As String, _
                                  ByVal description As String, _
                                  Optional ByVal newSheetLocation As String = "", _
                                  Optional ByVal newSheetDWTLocation As String = "", _
                                  Optional ByVal newSheetDWTLayout As String = "", _
                                  Optional ByVal promptForDWT As Boolean = False) _
                                  As AcSmSubset

        ' Create a subset with the provided name and description
        Dim subset As AcSmSubset = sheetSetDatabase.GetSheetSet(). _
                                       CreateSubset(name, description)

        ' Get the folder the sheet set is stored in
        Dim sheetSetFolder As String
        sheetSetFolder = Mid(sheetSetDatabase.GetFileName(), _
                             1, InStrRev(sheetSetDatabase.GetFileName(), "\"))

        ' Create a reference to a File Reference object
        Dim fileReference As IAcSmFileReference
        fileReference = subset.GetNewSheetLocation()

        ' Check to see if a path was provided, if not default 
        ' to the location of the sheet set
        If newSheetLocation <> "" Then
            fileReference.SetFileName(newSheetLocation)
        Else
            fileReference.SetFileName(sheetSetFolder)
        End If

        ' Set the location for new sheets added to the subset
        subset.SetNewSheetLocation(fileReference)

        ' Create a reference to a Layout Reference object
        Dim layoutReference As AcSmAcDbLayoutReference
        layoutReference = subset.GetDefDwtLayout

        ' Check to see that a default DWT location and name was provided
        If newSheetDWTLocation <> "" Then
            ' Set the template location and name of the layout 
            'for the Layout Reference object
            layoutReference.SetFileName(newSheetDWTLocation)
            layoutReference.SetName(newSheetDWTLayout)

            ' Set the Layout Reference for the subset
            subset.SetDefDwtLayout(layoutReference)
        End If

        ' Set the Prompt for Template option of the subset
        subset.SetPromptForDwt(promptForDWT)

        CreateSubset = subset
    End Function

    ' Used to add a sheet to a sheet set or subset
    ' Note: This function is dependent on a Default Template and Storage location
    ' being set for the sheet set or subset.
    Private Function AddSheet(ByVal component As IAcSmComponent, _
                              ByVal name As String, _
                              ByVal description As String, _
                              ByVal title As String, _
                              ByVal number As String) As AcSmSheet

        Dim sheet As AcSmSheet

        ' Check to see if the component is a sheet set or subset, 
        ' and create the new sheet based on the component's type
        If component.GetTypeName = "AcSmSubset" Then
            Dim subset As AcSmSubset = component
            sheet = subset.AddNewSheet(name, description)

            ' Add the sheet as the first one in the subset
            subset.InsertComponent(sheet, Nothing)
        Else
            sheet = component.GetDatabase().GetSheetSet().AddNewSheet(name, _
                                                                      description)

            ' Add the sheet as the first one in the sheet set
            component.GetDatabase().GetSheetSet().InsertComponent(sheet, Nothing)
        End If

        ' Set the number and title of the sheet
        sheet.SetNumber(number)
        sheet.SetTitle(title)

        AddSheet = sheet
    End Function


    ' Set the default properties of a sheet set
    Private Sub SetSheetSetDefaults(ByVal sheetSetDatabase As AcSmDatabase, _
                                    ByVal name As String, _
                                    ByVal description As String, _
                                    Optional ByVal newSheetLocation As String = "", _
                                    Optional ByVal newSheetDWTLocation As String = "", _
                                    Optional ByVal newSheetDWTLayout As String = "", _
                                    Optional ByVal promptForDWT As Boolean = False)

        ' Set the Name and Description for the sheet set
        sheetSetDatabase.GetSheetSet().SetName(name)
        sheetSetDatabase.GetSheetSet().SetDesc(description)

        ' Check to see if a Storage Location was provided
        If newSheetLocation <> "" Then
            ' Get the folder the sheet set is stored in
            Dim sheetSetFolder As String
            sheetSetFolder = Mid(sheetSetDatabase.GetFileName(), 1, _
                                 InStrRev(sheetSetDatabase.GetFileName(), "\"))

            ' Create a reference to a File Reference object
            Dim fileReference As IAcSmFileReference
            fileReference = sheetSetDatabase.GetSheetSet().GetNewSheetLocation()

            ' Set the default storage location based on the location of the sheet set
            fileReference.SetFileName(sheetSetFolder)

            ' Set the new Sheet location for the sheet set
            sheetSetDatabase.GetSheetSet().SetNewSheetLocation(fileReference)
        End If

        ' Check to see if a Template was provided
        If newSheetDWTLocation <> "" Then
            ' Set the Default Template for the sheet set
            Dim layoutReference As AcSmAcDbLayoutReference
            layoutReference = sheetSetDatabase.GetSheetSet().GetDefDwtLayout()

            ' Set the template location and name of the layout 
            ' for the Layout Reference object
            layoutReference.SetFileName(newSheetDWTLocation)
            layoutReference.SetName(newSheetDWTLayout)

            ' Set the Layout Reference for the sheet set
            sheetSetDatabase.GetSheetSet().SetDefDwtLayout(layoutReference)
        End If

        ' Set the Prompt for Template option of the subset
        sheetSetDatabase.GetSheetSet().SetPromptForDwt(promptForDWT)
    End Sub

    ' Import a sheet into a sheet set or subset
    Private Function ImportASheet(ByVal component As IAcSmComponent, _
                                  ByVal title As String, _
                                  ByVal description As String, _
                                  ByVal number As String, _
                                  ByVal fileName As String, _
                                  ByVal layout As String) As AcSmSheet

        Dim sheet As AcSmSheet

        ' Create a reference to a Layout Reference object
        Dim layoutReference As New AcSmAcDbLayoutReference
        layoutReference.InitNew(component)

        ' Set the layout and drawing file to use for the sheet
        layoutReference.SetFileName(fileName)
        layoutReference.SetName(layout)

        ' Import the sheet into the sheet set
        ' Check to see if the Component is a Subset or Sheet Set
        If component.GetTypeName = "AcSmSubset" Then
            Dim subset As AcSmSubset = component

            sheet = subset.ImportSheet(layoutReference)
            subset.InsertComponent(sheet, Nothing)
        Else
            Dim sheetSetDatabase As AcSmDatabase = component

            sheet = sheetSetDatabase.GetSheetSet().ImportSheet(layoutReference)
            sheetSetDatabase.GetSheetSet().InsertComponent(sheet, Nothing)
        End If

        ' Set the properties of the sheet
        sheet.SetDesc(description)
        sheet.SetTitle(title)
        sheet.SetNumber(number)

        ImportASheet = sheet
    End Function

    ' Set/create a custom sheet or sheet set property
    Private Sub SetCustomProperty(ByVal owner As IAcSmPersist, _
                                  ByVal propertyName As String, _
                                  ByVal propertyValue As Object, _
                                  ByVal sheetSetFlag As PropertyFlags)

        ' Create a reference to the Custom Property Bag
        Dim customPropertyBag As AcSmCustomPropertyBag

        If owner.GetTypeName() = "AcSmSheet" Then
            Dim sheet As AcSmSheet = owner
            customPropertyBag = sheet.GetCustomPropertyBag()
        Else
            Dim sheetSet As AcSmSheetSet = owner
            customPropertyBag = sheetSet.GetCustomPropertyBag()
        End If

        ' Create a reference to a Custom Property Value
        Dim customPropertyValue As AcSmCustomPropertyValue = New AcSmCustomPropertyValue()
        customPropertyValue.InitNew(owner)

        ' Set the flag for the property
        customPropertyValue.SetFlags(sheetSetFlag)

        ' Set the value for the property
        customPropertyValue.SetValue(propertyValue)

        ' Create the property
        customPropertyBag.SetProperty(propertyName, customPropertyValue)
    End Sub

    ' Synchronize the properties of a sheet with the sheet set
    Private Sub SyncProperties(ByVal sheetSetDatabase As IAcSmDatabase)
        ' Get the objects in the sheet set
        Dim enumerator As IAcSmEnumPersist
        enumerator = sheetSetDatabase.GetEnumerator()

        ' Get the first object in the Enumerator
        Dim item As IAcSmPersist
        item = enumerator.Next()

        ' Step through all the objects in the sheet set
        Do While Not item Is Nothing
            Dim sheet As IAcSmSheet = Nothing

            ' Check to see if the object is a sheet
            If item.GetTypeName() = "AcSmSheet" Then
                sheet = item

                ' Create a reference to the Property Enumerator for 
                ' the Custom Property Bag
                Dim enumeratorProperty As IAcSmEnumProperty
                enumeratorProperty = item.GetDatabase().GetSheetSet(). _
                GetCustomPropertyBag().GetPropertyEnumerator()

                ' Get the values from the Sheet Set to populate to the sheets
                Dim name As String = ""
                Dim customPropertyValue As AcSmCustomPropertyValue = Nothing

                ' Get the first property
                enumeratorProperty.Next(name, customPropertyValue)

                ' Step through each of the properties
                Do While Not customPropertyValue Is Nothing
                    ' Check to see if the property is for a sheet
                    If customPropertyValue.GetFlags() = _
                       PropertyFlags.CUSTOM_SHEET_PROP Then

                        SetCustomProperty(sheet, name, _
                                          customPropertyValue.GetValue(), _
                                          customPropertyValue.GetFlags())
                    End If

                    ' Get the next property
                    enumeratorProperty.Next(name, customPropertyValue)
                Loop
            End If

            ' Get the next Sheet
            item = enumerator.Next()
        Loop
    End Sub

    ' Register event handlers with the Sheet Set Manager, 
    ' database, and sheet set
    Private Sub AddEvents(ByVal sheetSetManager As IAcSmSheetSetMgr, _
                          ByVal sheetSetDatabase As IAcSmDatabase, _
                          ByVal sheetSet As IAcSmSheetSet)
        On Error Resume Next
        If Not eventHandler Is Nothing Then
            Exit Sub
        End If

        eventHandler = New MySSmEventHandler

        m_sheetSetManager = sheetSetManager
        m_sheetSetDatabase = sheetSetDatabase
        m_sheetSet = sheetSet

        ' Register an event for the Sheet Set Manager
        eventSSMCookie = m_sheetSetManager.Register(eventHandler)

        ' Register a database event
        eventDbCookie = m_sheetSetDatabase.Register(eventHandler)

        ' Register for sheet set event
        eventSSetCookie = m_sheetSet.Register(eventHandler)
    End Sub

    ' Remove the registered event handlers for the Sheet Set Manager, 
    ' database, and sheet set
    Private Sub RemoveEvents()
        On Error Resume Next
        If eventHandler Is Nothing Then
            Exit Sub
        End If

        m_sheetSetManager.Unregister(eventSSMCookie)
        m_sheetSetDatabase.Unregister(eventDbCookie)
        m_sheetSet.Unregister(eventSSetCookie)

        eventHandler = Nothing
    End Sub

    ' Add a sheet selection set
    Function AddSheetSelectionSet(ByVal sheetSetDatabase As AcSmDatabase, _
                                  ByVal name As String, _
                                  ByVal description As String) As IAcSmSheetSelSet

        ' Get the sheet selection sets for the sheet set
        Dim sheetSelSets As IAcSmSheetSelSets
        sheetSelSets = sheetSetDatabase.GetSheetSet().GetSheetSelSets()

        ' Add the new sheet selection set
        Dim sheetSelSet As IAcSmSheetSelSet = Nothing
        sheetSelSets.Add(name, description, sheetSelSet)

        ' Return the selection set added
        AddSheetSelectionSet = sheetSelSet
    End Function

    ' Add a drawing resource location to the sheet set
    Public Sub AddResourceFileLocation(ByVal sheetSetDatabase As AcSmDatabase, _
                                       ByVal resourceLoc As String)
        Dim resources As IAcSmResources
        Dim fileReference As New AcSmFileReference

        ' Create a reference to a resource location
        resources = sheetSetDatabase.GetSheetSet().GetResources()
        fileReference.InitNew(sheetSetDatabase)
        fileReference.SetFileName(resourceLoc)

        ' Add the drawing resource location
        resources.Add(fileReference)
    End Sub

    ' Add a callout block to the sheet set
    Public Sub AddCalloutBlock(ByVal sheetSetDatabase As AcSmDatabase, _
                               ByVal blockName As String, _
                               ByVal drawingFile As String)

        Dim calloutBlocks As IAcSmCalloutBlocks
        Dim calloutBlockReference As New AcSmAcDbBlockRecordReference

        ' Get the callout blocks for the sheet set
        calloutBlocks = sheetSetDatabase.GetSheetSet().GetCalloutBlocks()

        ' Create a new reference to a callout block and define the block to use
        calloutBlockReference.InitNew(sheetSetDatabase)
        calloutBlockReference.SetFileName(drawingFile)
        calloutBlockReference.SetName(blockName)

        ' Add the new block to the callout blocks for the sheet set
        calloutBlocks.Add(calloutBlockReference)
    End Sub

    ' Add a label block to the sheet set
    Public Sub AddLabelBlock(ByVal sheetSetDatabase As AcSmDatabase, _
                             ByVal blockName As String, _
                             ByVal drawingFile As String)

        Dim labelBlockReference As New AcSmAcDbBlockRecordReference

        ' Create a reference to a layout label block
        labelBlockReference.InitNew(sheetSetDatabase)
        labelBlockReference.SetFileName(drawingFile)
        labelBlockReference.SetName(blockName)

        ' Set the label block for the sheet set
        sheetSetDatabase.GetSheetSet().SetDefLabelBlk(labelBlockReference)
    End Sub

    ' Add a page setup override to the sheet set
    Public Sub AddPageSetupOverride(ByVal sheetSetDatabase As AcSmDatabase, _
                                    ByVal pageSetupName As String, _
                                    ByVal drawingFile As String)

        Dim fileReference As New AcSmFileReference

        ' Create a new reference to the page setup overrides to use
        fileReference.InitNew(sheetSetDatabase)
        fileReference.SetFileName(drawingFile)

        ' Set the drawing template that contains the page setup overrides
        sheetSetDatabase.GetSheetSet().SetAltPageSetups(fileReference)
    End Sub
End Class
